import argparse
import sys
from pyfiglet import Figlet
from tabulate import tabulate
from FCPpk.consts import COMPETITIONS
from FCPpk.FCP import get_games_by_country, get_live_games, get_top_todays_games


def main():

    parser = argparse.ArgumentParser(
        description="Track football games from the terminal",
        prog="FCP"
    )
    parser.add_argument("-t", default="default", help="reprsent time take two options ['now', 'today']", nargs='?', type=str)
    parser.add_argument("-c", default="default", help="Today's games in a specific country", nargs='?', type=str)
    args = parser.parse_args()

    figlet = Figlet()
    figlet.setFont = "big"

    games = filter_flags(args)
    if games == False:
         sys.exit(parser.print_help())
    elif games is None:
        sys.exit(figlet.renderText("Opps Try after 1 minute"))
    elif len(games) == 0:
        sys.exit(figlet.renderText("Nothing Now"))


    print(
        tabulate(
            games,
            headers="keys",
            tablefmt="mixed_outline",
            colalign=("center", "center", "center", "center"),
        )
    )


def filter_flags(args) -> bool | dict:

    if len(sys.argv) > 4 or len(sys.argv) < 2:
        return False

    time = args.t
    if time != "default":
        if args.t == "now":
            games = get_live_games()
        elif time == "today":
            games = get_top_todays_games()
        else:
            return False
    else:
        country = args.c
        if country not in COMPETITIONS:
            sys.exit("country doesn't exit visit country_list: https://github.com/code50/73303972/tree/main/CS50p/project#country-code")
        games = get_games_by_country(country)

    return games


if __name__ == "__main__":
    main()





"""

FC Python
~~~~~~~~~~

FCP is a python library to facilitate working with "www.api-football.com"
API docs : https://www.api-football.com/documentation-v3

    >>> import FCP
    >>> import tabulate
    >>> FCP.headers = {
        "x-rapidapi-key": "XxxXxXXxXxXxXxXxXXxXxXxX",
    }
    >>> games = FCP.get_live_game()
    >>> print(tabulate.tabulate(games))
    +-------------+-----------------+--------------+----------------+
    |   TIME      |     TEAM_A      |     VS       |      TEAM_B    |
    +-------------+-----------------+--------------+----------------+
    |     51'     |    Liverpool    |    2 - 1     |   Chelsea      |
    |     33'     |    Newcastle    |    1 - 0     |   West Ham     |
    |     14'     |     Arsenal     |    1 - 2     |   Man City     |
    +-------------+-----------------+--------------+----------------+

 ______ _____ ______      ______      _   _
|  ___/  __ \| ___ \  _  | ___ \    | | | |
| |_  | /  \/| |_/ / (_) | |_/ /   _| |_| |__   ___  _ __
|  _| | |    |  __/      |  __/ | | | __| '_ \ / _ \| '_ \\
| |   | \__/\| |      _  | |  | |_| | |_| | | | (_) | | | |
\_|    \____/\_|     (_) \_|   \__, |\__|_| |_|\___/|_| |_|
                                __/ |
                               |___/
______          _   _           _ _   _____  _     _   _______
|  ___|        | | | |         | | | /  __ \| |   | | | | ___ \\
| |_ ___   ___ | |_| |__   __ _| | | | /  \/| |   | | | | |_/ /
|  _/ _ \ / _ \| __| '_ \ / _` | | | | |    | |   | | | | ___ \\
| || (_) | (_) | |_| |_) | (_| | | | | \__/\| |___| |_| | |_/ //
\_| \___/ \___/ \__|_.__/ \__,_|_|_|  \____/\_____/\___/\____//


"""

import os
import sys
import requests
from datetime import date
from FCPpk.consts import TOP_COMPETITIONS, COMPETITIONS, TIMEZONES, TODAYS_TOP


# "d6945347736976d8f423047935da8706"

api_key = os.environ.get("API_KEY")
if not api_key:
    sys.exit("Export API_KEY first 🙏")

time_zone = os.environ.get("TIMEZONE")
if not time_zone:
    sys.exit("Export TIMEZONE first 🙏")


headers = {
    "x-rapidapi-key": api_key
}

parameters = {
    "timezone": time_zone
}


def get_games(URL: str) -> None | list:

    """
    API docs : https://www.api-football.com/documentation-v3#tag/Fixtures
    :params URL:  where host is 'https://v3.football.api-sports.io/fixtures'
    :type URL: str
    :raise ValueErorr: if timezone does not exist, API_KEY,  and Invalid URL
    :return: None if the api was unsuccessful to provied data
    :return: Games in a the given URL
    :rtype: dict|None:
    """

    if not isvalid_URL(URL):
        raise ValueError("Invalid URL")

    timezone = parameters["timezone"]
    if timezone not in TIMEZONES:
        raise ValueError(f'{parameters["timezone"]} does not exist')

    try:
        response = requests.get(URL, headers=headers)
        response.raise_for_status()
        GAMES = response.json()
    except requests.RequestException:
        return None
    else:
        if "token" in GAMES["errors"]:
            raise ValueError(f'API_KEY is not correct')
        if len(GAMES["errors"]) != 0:
            return None

    today_games = []
    try:
        for game in GAMES["response"]:
            today_games.append(
                style(
                    {
                        "team_a": game["teams"]["home"]["name"],
                        "goals_a": game["goals"]["home"],
                        "team_b": game["teams"]["away"]["name"],
                        "goals_b": game["goals"]["away"],
                        "time": game["fixture"]["status"]["elapsed"],
                        "status": game["fixture"]["status"]["long"],
                        "date": game["fixture"]["date"].split("T")[1][0:5],
                        # penalties after extratime
                        "penalty_b": game["score"]["penalty"]["away"],
                        "penalty_a": game["score"]["penalty"]["home"],
                    }
                )
            )
    except (KeyError, TypeError, ValueError):
        return None

    return today_games


def get_competitions_ids(competitions: dict) -> str:

    """ helper function return competitions' ids as string """

    s = ""
    for country in competitions:
        for competition in competitions[country]:
            s = f"{s}-{str(competition)}"
    return s[1:]


def get_games_by_country(country: str) -> None | list:

    """
    API docs : https://www.api-football.com/documentation-v3#tag/Fixtures
    :params country: abbreviation of the country name
    :type country: str
    :raise ValueErorr: if country doesn't exist
    :return: None if the api was unsuccessful to provied data
    :return: Games in the given country
    :rtype: dict|None:
    """

    timezone = parameters["timezone"]
    today = date.today()

    if not country.lower() in COMPETITIONS:
        raise ValueError(f"{country} doesn't exist")

    # the season is always a year late than the current one
    season = today.year - 1
    competitions = COMPETITIONS[country.lower()]

    games = []
    for competition in competitions:
        URL = f"https://v3.football.api-sports.io/fixtures?league={competition}&season={season}&date={today}&timezone={timezone}"
        matches = get_games(URL)
        if matches is None:
            return None
        games.extend(matches)
    return games


def get_live_games() -> list | None:

    """
    API docs : https://www.api-football.com/documentation-v3#tag/Fixtures
    :return: None if the api was unsuccessful to provied data
    :return: live games
    :rtype: dict|None:

    """

    ids = get_competitions_ids(TOP_COMPETITIONS)
    URL = f"https://v3.football.api-sports.io/fixtures?live={ids}&timezone={parameters['timezone']}"
    games = get_games(URL)
    if games is None:
        return None
    matches = []
    for game in games:
        matches.append(game)
    return matches


def get_top_todays_games() -> list | None:

    """
    API docs : https://www.api-football.com/documentation-v3#tag/Fixtures
    :raise requests.RequestException: if request was unsuccessful
    :return: dict of Today's matches OR None if the api was unsuccessful to provied data
    :rtype: dict|None:

    NOTE: it's not recommended using this function if you have free plan
    """

    timezone = parameters["timezone"]
    # the season is always a year late than the current one
    today = date.today()
    season = today.year - 1
    games = []

    for country in TODAYS_TOP:
        for competition in TODAYS_TOP[country]:
            URL = f"https://v3.football.api-sports.io/fixtures?league={competition}&season={season}&date={today}&timezone={timezone}"
            matches = get_games(URL)
            if matches is None:
                return None
            games.extend(matches)
    return games


def isvalid_URL(url) -> bool:

    HOST = "https://v3.football.api-sports.io/fixtures"
    try:
        host, params = url.split("?")
        if not params:
            return False
        return host == HOST
    except:
        return False


def style(game: dict) -> dict:

    """ b """

    game_info = {
        "TIME": game["status"],
        "TEAM_A": game["team_a"],
        "VS": "-  -",
        "TEAM_B": game["team_b"],
    }

    if "Not Started" == game["status"]:
        game_info["VS"] = game["date"]
    elif "Finished" in game["status"]:
        game_info["VS"] = f"{game['goals_a']} - {game['goals_b']}"
    elif game["time"] is not None:
        game_info["TIME"] = f"{game['time']}'"
        game_info["VS"] = f"{game['goals_a']} - {game['goals_b']}"
    if game["penalty_a"] is not None:
        game_info["VS"] = f'{game_info["VS"]} P({game["penalty_a"]}-{game["penalty_b"]})'


    return game_info




"""  FCP constants  """

TOP_COMPETITIONS = {
    "Europe": [2, 3, 4, 5],  #  Champions League,  Europa League, Euro Championship
    "eng": [39, 40, 45, 48],  # PREMIER LEAGUE,  Championship,  Fa Cup, League Cup
    "Africa": [6, 12, 19, 20],  # Cup of Nations, Champions League , Nations Championship,  Confederation Cup
    "mar": [200],  # Botola PRO INWI
    "esp": [140, 556, 143],  # LA LIGA,  Super Cup, Copa del Rey
    "itl": [135, 137, 547],  # SERIE A,  Coppa Itali , Super Cup
    "fr": [61, 66],  # Ligue 1 ,  Coupe de France
    "de": [78, 81, 529],  # BUNDESLIGA 1, DFB Pokal, Super Cup
}


TODAYS_TOP = {
    "Europe": [2],  #  Champions League,  Europa League
    "eng": [39],  # PREMIER LEAGUE
    "Africa": [6, 12],  # Cup of Nations, Champions League
    "esp": [140, 143],  # LA LIGA, Copa del Rey
    "itl": [135],  # SERIE A
    "de": [78],  # BUNDESLIGA 1
    "fr": [61],  # Ligue 1
}


COMPETITIONS = {
    "mar": [200],  # Botola PRO INWI
    "tun": [202],  # Tunisia Ligue,
    "tur": [203, 204, 206],  # turkey Süper Lig, 1-Lig, Cup
    "egp": [233],  #  Egypt Premier League
    "eng": [39, 40, 41, 45, 48,], # PREMIER LEAGUE,  Championship, League 1 , Fa Cup, League Cup
    "fr": [61, 62, 66],  # Ligue 1 ,  Ligue 2, Coupe de France
    "de": [78, 79, 81, 529],  # BUNDESLIGA 1, BUNDESLIGA 2, DFB Pokal Super Cup
    "esp": [140, 141, 556, 143],  # LA LIGA, Segunda División, Super Cup, Copa del Rey
    "itl": [135, 136, 137, 547],  # SERIE A, SERIE B , Coppa Itali, Super Cup
    "nld": [88, 89, 90, 543],  #  Eredivisie League, Eerste Divisie, KNVB Beker
    "ptr": [ 94, 95, 96, 97],  # Primeira Liga, Segunda Liga, Taça de Portugal, Taça da Liga
    "blg": [144, 145, 147],  # Jupiler Pro League,  Challenger Pro League, Cup
    "usa": [253, 255],  #  Major League Soccer, USL Championship
    "arg" : [128], # Liga Profesional Argentina
    "europe": [2, 3, 4, 5],  #  Champions League,  Europa League, Euro Championship
    "africa": [ 6, 12, 19, 20],  # Cup of Nations, Champions League , Nations Championship,  Confederation Cup
    "asia": [7, 17, 18],  # Asian Cup ,  AFC Champions League
    "america": [9, 12],
    "inter": [1, 10, 15],  # FIFA World Cup, Friendlies, FIFA Club World Cup
}


TIMEZONES = [
    "Africa/Abidjan",
    "Africa/Accra",
    "Africa/Addis_Ababa",
    "Africa/Algiers",
    "Africa/Asmara",
    "Africa/Bamako",
    "Africa/Bangui",
    "Africa/Banjul",
    "Africa/Bissau",
    "Africa/Blantyre",
    "Africa/Brazzaville",
    "Africa/Bujumbura",
    "Africa/Cairo",
    "Africa/Casablanca",
    "Africa/Ceuta",
    "Africa/Conakry",
    "Africa/Dakar",
    "Africa/Dar_es_Salaam",
    "Africa/Djibouti",
    "Africa/Douala",
    "Africa/El_Aaiun",
    "Africa/Freetown",
    "Africa/Gaborone",
    "Africa/Harare",
    "Africa/Johannesburg",
    "Africa/Juba",
    "Africa/Kampala",
    "Africa/Khartoum",
    "Africa/Kigali",
    "Africa/Kinshasa",
    "Africa/Lagos",
    "Africa/Libreville",
    "Africa/Lome",
    "Africa/Luanda",
    "Africa/Lubumbashi",
    "Africa/Lusaka",
    "Africa/Malabo",
    "Africa/Maputo",
    "Africa/Maseru",
    "Africa/Mbabane",
    "Africa/Mogadishu",
    "Africa/Monrovia",
    "Africa/Nairobi",
    "Africa/Ndjamena",
    "Africa/Niamey",
    "Africa/Nouakchott",
    "Africa/Ouagadougou",
    "Africa/Porto-Novo",
    "Africa/Sao_Tome",
    "Africa/Tripoli",
    "Africa/Tunis",
    "Africa/Windhoek",
    "America/Adak",
    "America/Anchorage",
    "America/Anguilla",
    "America/Antigua",
    "America/Araguaina",
    "America/Argentina/Buenos_Aires",
    "America/Argentina/Catamarca",
    "America/Argentina/Cordoba",
    "America/Argentina/Jujuy",
    "America/Argentina/La_Rioja",
    "America/Argentina/Mendoza",
    "America/Argentina/Rio_Gallegos",
    "America/Argentina/Salta",
    "America/Argentina/San_Juan",
    "America/Argentina/San_Luis",
    "America/Argentina/Tucuman",
    "America/Argentina/Ushuaia",
    "America/Aruba",
    "America/Asuncion",
    "America/Atikokan",
    "America/Bahia",
    "America/Bahia_Banderas",
    "America/Barbados",
    "America/Belem",
    "America/Belize",
    "America/Blanc-Sablon",
    "America/Boa_Vista",
    "America/Bogota",
    "America/Boise",
    "America/Cambridge_Bay",
    "America/Campo_Grande",
    "America/Cancun",
    "America/Caracas",
    "America/Cayenne",
    "America/Cayman",
    "America/Chicago",
    "America/Chihuahua",
    "America/Costa_Rica",
    "America/Creston",
    "America/Cuiaba",
    "America/Curacao",
    "America/Danmarkshavn",
    "America/Dawson",
    "America/Dawson_Creek",
    "America/Denver",
    "America/Detroit",
    "America/Dominica",
    "America/Edmonton",
    "America/Eirunepe",
    "America/El_Salvador",
    "America/Fort_Nelson",
    "America/Fortaleza",
    "America/Glace_Bay",
    "America/Godthab",
    "America/Goose_Bay",
    "America/Grand_Turk",
    "America/Grenada",
    "America/Guadeloupe",
    "America/Guatemala",
    "America/Guayaquil",
    "America/Guyana",
    "America/Halifax",
    "America/Havana",
    "America/Hermosillo",
    "America/Indiana/Indianapolis",
    "America/Indiana/Knox",
    "America/Indiana/Marengo",
    "America/Indiana/Petersburg",
    "America/Indiana/Tell_City",
    "America/Indiana/Vevay",
    "America/Indiana/Vincennes",
    "America/Indiana/Winamac",
    "America/Inuvik",
    "America/Iqaluit",
    "America/Jamaica",
    "America/Juneau",
    "America/Kentucky/Louisville",
    "America/Kentucky/Monticello",
    "America/Kralendijk",
    "America/La_Paz",
    "America/Lima",
    "America/Los_Angeles",
    "America/Lower_Princes",
    "America/Maceio",
    "America/Managua",
    "America/Manaus",
    "America/Marigot",
    "America/Martinique",
    "America/Matamoros",
    "America/Mazatlan",
    "America/Menominee",
    "America/Merida",
    "America/Metlakatla",
    "America/Mexico_City",
    "America/Miquelon",
    "America/Moncton",
    "America/Monterrey",
    "America/Montevideo",
    "America/Montserrat",
    "America/Nassau",
    "America/New_York",
    "America/Nipigon",
    "America/Nome",
    "America/Noronha",
    "America/North_Dakota/Beulah",
    "America/North_Dakota/Center",
    "America/North_Dakota/New_Salem",
    "America/Ojinaga",
    "America/Panama",
    "America/Pangnirtung",
    "America/Paramaribo",
    "America/Phoenix",
    "America/Port-au-Prince",
    "America/Port_of_Spain",
    "America/Porto_Velho",
    "America/Puerto_Rico",
    "America/Punta_Arenas",
    "America/Rainy_River",
    "America/Rankin_Inlet",
    "America/Recife",
    "America/Regina",
    "America/Resolute",
    "America/Rio_Branco",
    "America/Santarem",
    "America/Santiago",
    "America/Santo_Domingo",
    "America/Sao_Paulo",
    "America/Scoresbysund",
    "America/Sitka",
    "America/St_Barthelemy",
    "America/St_Johns",
    "America/St_Kitts",
    "America/St_Lucia",
    "America/St_Thomas",
    "America/St_Vincent",
    "America/Swift_Current",
    "America/Tegucigalpa",
    "America/Thule",
    "America/Thunder_Bay",
    "America/Tijuana",
    "America/Toronto",
    "America/Tortola",
    "America/Vancouver",
    "America/Whitehorse",
    "America/Winnipeg",
    "America/Yakutat",
    "America/Yellowknife",
    "Antarctica/Casey",
    "Antarctica/Davis",
    "Antarctica/DumontDUrville",
    "Antarctica/Macquarie",
    "Antarctica/Mawson",
    "Antarctica/McMur",
    "Antarctica/PalmerAnta",
    "Antarctica/Rothera",
    "Antarctica/Syowa",
    "Antarctica/Troll",
    "Antarctica/Vostok",
    "Arctic/Longyearbyen",
    "Asia/Aden",
    "Asia/Almaty",
    "Asia/Amman",
    "Asia/Anadyr",
    "Asia/Aqtau",
    "Asia/Aqtobe",
    "Asia/Ashgabat",
    "Asia/Atyrau",
    "Asia/Baghdad",
    "Asia/Bahrain",
    "Asia/Baku",
    "Asia/Bangkok",
    "Asia/Barnaul",
    "Asia/Beirut",
    "Asia/Bishkek",
    "Asia/Brunei",
    "Asia/Calcutta",
    "Asia/Chita",
    "Asia/Choibalsan",
    "Asia/Colombo",
    "Asia/Damascus",
    "Asia/Dhaka",
    "Asia/Dili",
    "Asia/Dubai",
    "Asia/Dushanbe",
    "Asia/Famagusta",
    "Asia/Gaza",
    "Asia/Hebron",
    "Asia/Ho_Chi_Minh",
    "Asia/Hong_Kong",
    "Asia/Hovd",
    "Asia/Irkutsk",
    "Asia/Jakarta",
    "Asia/Jayapura",
    "Asia/Jerusalem",
    "Asia/Kabul",
    "Asia/Kamchatka",
    "Asia/Karachi",
    "Asia/Kathmandu",
    "Asia/Khandyga",
    "Asia/Kolkata",
    "Asia/Krasnoyarsk",
    "Asia/Kuala_Lumpur",
    "Asia/Kuching",
    "Asia/Kuwait",
    "Asia/Macau",
    "Asia/Magadan",
    "Asia/Makassar",
    "Asia/Manila",
    "Asia/Muscat",
    "Asia/Nicosia",
    "Asia/Novokuznetsk",
    "Asia/Novosibirsk",
    "Asia/Omsk",
    "Asia/Oral",
    "Asia/Phnom_Penh",
    "Asia/Pontianak",
    "Asia/Pyongyang",
    "Asia/Qatar",
    "Asia/Qostanay",
    "Asia/Qyzylorda",
    "Asia/Riyadh",
    "Asia/Saigon",
    "Asia/Sakhalin",
    "Asia/Samarkand",
    "Asia/Seoul",
    "Asia/Shanghai",
    "Asia/Singapore",
    "Asia/Srednekolymsk",
    "Asia/Taipei",
    "Asia/Tashkent",
    "Asia/Tbilisi",
    "Asia/Tehran",
    "Asia/Thimphu",
    "Asia/Tokyo",
    "Asia/Tomsk",
    "Asia/Ulaanbaatar",
    "Asia/Urumqi",
    "Asia/Ust-Nera",
    "Asia/Vientiane",
    "Asia/Vladivostok",
    "Asia/Yakutsk",
    "Asia/Yangon",
    "Asia/Yekaterinburg",
    "Asia/Yerevan",
    "Atlantic/Azores",
    "Atlantic/Bermuda",
    "Atlantic/Canary",
    "Atlantic/Cape_Verde",
    "Atlantic/Faroe",
    "Atlantic/Madeira",
    "Atlantic/Reykjavik",
    "Atlantic/South_Georgia",
    "Atlantic/St_Helena",
    "Atlantic/Stanley",
    "Australia/Adelaide",
    "Australia/Brisbane",
    "Australia/Broken_Hill",
    "Australia/Currie",
    "Australia/Darwin",
    "Australia/Eucla",
    "Australia/Hobart",
    "Australia/Lindeman",
    "Australia/Lord_Howe",
    "Australia/Melbourne",
    "Australia/Perth",
    "Australia/Sydney",
    "Europe/Amsterdam",
    "Europe/Andorra",
    "Europe/Astrakhan",
    "Europe/Athens",
    "Europe/Belgrade",
    "Europe/Berlin",
    "Europe/Bratislava",
    "Europe/Brussels",
    "Europe/Bucharest",
    "Europe/Budapest",
    "Europe/Busingen",
    "Europe/Chisinau",
    "Europe/Copenhagen",
    "Europe/Dublin",
    "Europe/Gibraltar",
    "Europe/Guernsey",
    "Europe/Helsinki",
    "Europe/Isle_of_Man",
    "Europe/Istanbul",
    "Europe/Jersey",
    "Europe/Kaliningrad",
    "Europe/Kiev",
    "Europe/Kirov",
    "Europe/Lisbon",
    "Europe/Ljubljana",
    "Europe/London",
    "Europe/Luxembourg",
    "Europe/Madrid",
    "Europe/Malta",
    "Europe/Mariehamn",
    "Europe/Minsk",
    "Europe/Monaco",
    "Europe/Moscow",
    "Europe/Oslo",
    "Europe/Paris",
    "Europe/Podgorica",
    "Europe/Prague",
    "Europe/Riga",
    "Europe/Rome",
    "Europe/Samara",
    "Europe/San_Marino",
    "Europe/Sarajevo",
    "Europe/Saratov",
    "Europe/Simferopol",
    "Europe/Skopje",
    "Europe/Sofia",
    "Europe/Stockholm",
    "Europe/Tallinn",
    "Europe/Tirane",
    "Europe/Ulyanovsk",
    "Europe/Uzhgorod",
    "Europe/Vaduz",
    "Europe/Vatican",
    "Europe/Vienna",
    "Europe/Vilnius",
    "Europe/Volgograd",
    "Europe/Warsaw",
    "Europe/Zagreb",
    "Europe/Zaporozhye",
    "Europe/Zurich",
    "Indian/Antananarivo",
    "Indian/Chagos",
    "Indian/Christmas",
    "Indian/Cocos",
    "Indian/Comoro",
    "Indian/Kerguelen",
    "Indian/Mahe",
    "Indian/Maldives",
    "Indian/Mauritius",
    "Indian/Mayotte",
    "Indian/Reunion",
    "Pacific/Apia",
    "Pacific/Auckland",
    "Pacific/Bougainville",
    "Pacific/Chatham",
    "Pacific/Chuuk",
    "Pacific/Easter",
    "Pacific/Efate",
    "Pacific/Enderbury",
    "Pacific/Fakaofo",
    "Pacific/Fiji",
    "Pacific/Funafuti",
    "Pacific/Galapagos",
    "Pacific/Gambier",
    "Pacific/Guadalcanal",
    "Pacific/Guam",
    "Pacific/Honolulu",
    "Pacific/Kiritimati",
    "Pacific/Kosrae",
    "Pacific/Kwajalein",
    "Pacific/Majuro",
    "Pacific/Marquesas",
    "Pacific/Midway",
    "Pacific/Nauru",
    "Pacific/Niue",
    "Pacific/Norfolk",
    "Pacific/Noumea",
    "Pacific/Pago_Pago",
    "Pacific/Palau",
    "Pacific/Pitcairn",
    "Pacific/Pohnpei",
    "Pacific/Port_Moresby",
    "Pacific/Rarotonga",
    "Pacific/Saipan",
    "Pacific/Tahiti",
    "Pacific/Tarawa",
    "Pacific/Tongatapu",
    "Pacific/Wake",
    "Pacific/Wallis",
]